//
//
// hello.ftdi.44.echo.c
//
// 115200 baud FTDI character echo, with flash string
//
// set lfuse to 0x7E for 20 MHz xtal
//
// Neil Gershenfeld
// 12/8/10
//
// (c) Massachusetts Institute of Technology 2010
// Permission granted for experimental and personal use;
// license for commercial sale available from MIT.
//

#include <avr/io.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

#define output(directions,pin) (directions |= pin) // set port direction for output
#define set(port,pin) (port |= pin) // set port pin
#define clear(port,pin) (port &= (~pin)) // clear port pin
#define pin_test(pins,pin) (pins & pin) // test for port pin
#define bit_test(byte,bit) (byte & (1 << bit)) // test for bit set
#define bit_delay_time 8.5 // bit delay for 115200 with overhead
#define bit_delay() _delay_us(bit_delay_time) // RS232 bit delay
#define half_bit_delay() _delay_us(bit_delay_time/2) // RS232 half bit delay
#define char_delay() _delay_ms(10) // char delay

#define serial_port PORTA
#define serial_direction DDRA
#define serial_pins PINA
#define serial_pin_in (1 << PA0)
#define serial_pin_out (1 << PA1)

#define max_buffer 25

#define PWM_port PORTA
#define PWM_pin (1 << PA6)
#define PWM_direction DDRA

void get_char(volatile unsigned char *pins, unsigned char pin, char *rxbyte) {
   //
   // read character into rxbyte on pins pin
   //    assumes line driver (inverts bits)
   //
   *rxbyte = 0;
   while (pin_test(*pins,pin))
      //
      // wait for start bit
      //
      ;
   //
   // delay to middle of first data bit
   //
   half_bit_delay();
   bit_delay();
   //
   // unrolled loop to read data bits
   //
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 0);
   else
      *rxbyte |= (0 << 0);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 1);
   else
      *rxbyte |= (0 << 1);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 2);
   else
      *rxbyte |= (0 << 2);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 3);
   else
      *rxbyte |= (0 << 3);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 4);
   else
      *rxbyte |= (0 << 4);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 5);
   else
      *rxbyte |= (0 << 5);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 6);
   else
      *rxbyte |= (0 << 6);
   bit_delay();
   if pin_test(*pins,pin)
      *rxbyte |= (1 << 7);
   else
      *rxbyte |= (0 << 7);
   //
   // wait for stop bit
   //
   bit_delay();
   half_bit_delay();
   }

void put_char(volatile unsigned char *port, unsigned char pin, char txchar) {
   //
   // send character in txchar on port pin
   //    assumes line driver (inverts bits)
   //
   // start bit
   //
   clear(*port,pin);
   bit_delay();
   //
   // unrolled loop to write data bits
   //
   if bit_test(txchar,0)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,1)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,2)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,3)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,4)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,5)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,6)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   if bit_test(txchar,7)
      set(*port,pin);
   else
      clear(*port,pin);
   bit_delay();
   //
   // stop bit
   //
   set(*port,pin);
   bit_delay();
   //
   // char delay
   //
   bit_delay();
   }

void put_string(volatile unsigned char *port, unsigned char pin, char *str) {
   //
   // print a null-terminated string
   //
   static int index;
   index = 0;
   do {
      put_char(port, pin, str[index]);
      ++index;
      } while (str[index] != 0);
   }

int initialize_led_switch(void){
   DDRA |= (1<<PA3);		// LED is an output and located at PA3
   PORTA |= (1<<PA2);		// activate the internal resistor for the switch on PA2
}

int main(void) {
   //
   // main
   //
   static char chr;
   static char mode;

   // FTDI

   //
   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // initialize output pins
   //
   set(serial_port, serial_pin_out);
   output(serial_direction, serial_pin_out);

   // initialize led and switch
   initialize_led_switch();


   // PWM

   // set clock divider to /1
   //
   CLKPR = (1 << CLKPCE);
   CLKPR = (0 << CLKPS3) | (0 << CLKPS2) | (0 << CLKPS1) | (0 << CLKPS0);
   //
   // set up timer 1
   //
   TCCR1A = (1 << COM1A1) | (0 << COM1A0); // clear OC1A on compare match
   TCCR1B = (0 << CS12) | (1 << CS11) | (0 << CS10) | (1 << WGM13); // prescaler /8, phase and frequency correct PWM, ICR1 TOP
   ICR1 = 25000; // 20 ms frequency
   //
   // set PWM pin to output
   //
   clear(PWM_port, PWM_pin);
   output(PWM_direction, PWM_pin);
   //
   //
   // main loop
   //
   while (1) {
      get_char(&serial_pins, serial_pin_in, &mode);
      put_string(&serial_port, serial_pin_out, "You pressed: ");
      put_char(&serial_port, serial_pin_out, mode);
      put_char(&serial_port, serial_pin_out, 10); // new line
      if ( mode == '0' ) {
      	 OCR1A = 1250+(0*125);
         put_string(&serial_port, serial_pin_out, "0 \n");
         }
      else if ( mode == '1' ) {
      	 OCR1A = 1250+(1*125);
         put_string(&serial_port, serial_pin_out, "10 \n");
         }
      else if ( mode == '2' ) {
      	 OCR1A = 1250+(2*125);
         put_string(&serial_port, serial_pin_out, "20 \n");
         }
      else if ( mode == '3' ) {
      	 OCR1A = 1250+(3*125);
         put_string(&serial_port, serial_pin_out, "30 \n");
         }
      else if ( mode == '4' ) {
      	 OCR1A = 1250+(4*125);
         put_string(&serial_port, serial_pin_out, "40 \n");
         }
      else if ( mode == '5' ) {
      	 OCR1A = 1250+(5*125);
         put_string(&serial_port, serial_pin_out, "50 \n");
         }
      else if ( mode == '6' ) {
      	 OCR1A = 1250+(6*125);
         put_string(&serial_port, serial_pin_out, "60 \n");
         }
      else if ( mode == '7' ) {
      	 OCR1A = 1250+(7*125);
         put_string(&serial_port, serial_pin_out, "70 \n");
         }
      else if ( mode == '8' ) {
      	 OCR1A = 1250+(8*125);
         put_string(&serial_port, serial_pin_out, "80 \n");
         }
      else if ( mode == '9' ) {
      	 OCR1A = 1250+(9*125);
         put_string(&serial_port, serial_pin_out, "90 \n");
         }
      else if ( mode == 'f' ) {
      	 OCR1A = 1250+(10*125);
         put_string(&serial_port, serial_pin_out, "100 \n");
         }
      }
   }
